close all
clear all
clc

% addpath('W:\DIV1\STRATEGYTEAM\MISC\Ben\CompEcon\CEtools')
% addpath(genpath('C:\Users\986836\Desktop\CompEcon'))
% addpath(genpath('/Users/benjamindnelson/Documents/Benj/Work/CompEcon')) 

addpath(genpath('P\Tools')) ;


lqr=[0,1];
trp=[0,1];
xout=[];
spathout=[];
xpathout=[];
resout=[];
vout=[];
tauanaout=[];

for i=1:2
for j=1:2

 
% set params
alpha   = 0.3    ;
beta    = 0.80   ;
   pi   = 0.075  ;
lambda0 = 0.60   ;
lambda1 = 0.30   ; % no endogeneity if lambda1=0
lambda2 = 0.05   ; % default is lambda2=1
lambdab = 0.95   ;
    xb  = 0.775  ;       
   liq  = lqr(i) ;
   trap = trp(j) ;   
   
% check sum of lambda0 and lambda1 is less than unity
if lambda0+lambda1>1
    disp('lambda0+lambda1 is > 1');break;
end

% discretise the shock
nshocks = 50 ;
sigma = 0.015 ; % 0.04 for trap is okay, 0.015 for no trap is okay
mu    = 1 ;
% [e,w]=qnwlogn(nshocks,0,sigma^2) ; % e are the shocks; w are the weights
[e,w]=qnwnorm(nshocks,mu,sigma^2) ; % e are the shocks; w are the weights

% for liquidation terms, need to compute E[x^alpha]
draws=10000;
shks=mu+sigma*randn(draws,1);
Exalpha = mean( shks.^alpha ) ;
sigmaalpha=std(shks.^alpha);

% check solvency: e(1) is the lower bound on the shock distribution
if e(1)^alpha < (lambda0+lambda1)*Exalpha
    disp('Warning: banks can be insolvent with this shock distribution'); break;
end

% steady state guess
% estar=exp(sigma^2/2); % for log normal
estar=mu;
taustar = 1-(1/lambda0)*(2*alpha*beta - (1+alpha*beta)*pi)/(beta*(1-alpha*beta)) ;
kstar=( ((pi+beta*(1-taustar)*lambda0)/(1+beta*(1-taustar)*lambda0))*(1-alpha) )^(1/(1-alpha)) ;

% approximation details
n=20; % 20
% kmin=0.00*kstar; 
kmin=0.01; 
kmax=1.90*kstar;
fspace=fundefn('cheb',n,kmin,kmax);
knodes=funnode(fspace);

% check what the tau constraint looks like for this calibration
        kgrid=kmin:0.01:kmax;
        tmin=[];
        options=optimset('Display','off');
        for j=1:length(kgrid)
        ll=lambda0+lambda1.*(kgrid(j)./(lambda2+kgrid(j)));    
        zz = fsolve(@(t) ((pi+beta.*(1-t).*ll).^alpha).*((1+beta.*(1-t).*ll).^(1-alpha)).*Exalpha ...
               -((pi+beta.*(1-t).*lambdab).^alpha).*((1+beta.*(1-t).*lambdab).^(1-alpha)).*(xb^alpha),0.2,options) ;
        tmin=[tmin zz];
        end
%         figure, plot(kgrid,tmin), xlabel('k'),ylabel('\tau to avoid trap');
        clear tmin;
        
% check a steady state trap exists without policy

kbstar = ( xb*(1-alpha)*(pi+beta*lambdab)/(1+beta*lambdab) )^(1/(1-alpha));
lendo= lambda0 + lambda1.*( kbstar ./ ( lambda2 + kbstar ) ) ;
lhs = ((pi+beta*lendo)^alpha)*((1+beta*lendo)^(1-alpha))*Exalpha ;
rhs = ((pi+beta*lambdab)^alpha)*((1+beta*lambdab)^(1-alpha))*(xb^alpha) ;
if lhs < rhs
    disp(' ');
    disp('Trap exists at current calibration');
    disp(' ');
else
    disp('No trap exists -> change calibration')
    break
end

% model structure
model.func = 'traps9';
model.discount=beta;
model.e=e;
model.w=w;
model.params={alpha beta pi lambda0 lambda1 lambda2 Exalpha lambdab xb liq trap};

[vlq,xlq]=lqapprox(model,knodes,kstar,taustar,estar);

% solve model
[c,s,v,x,resid]=dpsolve(model,fspace,knodes,vlq,xlq);

% get analytical results for comparison
if trap==0
    tauana=[];
%     tauana=NaN(1,rows(s));
    opt=optimset('Display','off');
    for ii=1:length(s)
       tauana(ii)=fsolve(@(tau) foctau(alpha,beta,pi,lambda0,lambda1,lambda2,s(ii),tau,liq,Exalpha,mu,sigma),0.25,opt) ; 
    end
else
    tauana=NaN(1,rows(s));    
end


%% OUTPUTS
%   c         : value function approximation basis coefficients
%   scoord    : residual evaluation coordinates (cell array for ds>1)
%   v         : value function at evaluation points
%   x         : optimal action at evaluation points
%   resid     : Bellman equation residuals at evaluation points

% compute an optimal path
nyrs=50; % period
npath=10000; % number of paths
sinit=kstar*ones(npath,1); % initial conditions for the simulation
[spath,xpath]=dpsimul(model,sinit,nyrs,s,x);

% tt=0:1:nyrs; %  time grid

% compute st st distribution of wealth
% smooth=3; % smoothing param
% nnodes=10; % number of nodes in distribution approximation
% [ss,pip,xx]=dpstst(model,smooth,nnodes,s,x);
% 
% figure, plot(ss,pip)

% subplot(2,2,3)
% hold all
% plot(tt,median(spath(:,1:end))','b-')
% plot(tt,prctile(spath(:,1:end),5)','b--')
% plot(tt,prctile(spath(:,1:end),95)','b--')
% xlabel('Years'),ylabel('Capital, k_t'), 
% title('(c)')
% legend('median','Location','SouthEast')
% box on
% 
% subplot(2,2,4)
% hold all
% plot(tt,median(xpath(:,1:end))','b-')
% plot(tt,prctile(xpath(:,1:end),5)','b--')
% plot(tt,prctile(xpath(:,1:end),95)','b--')
% xlabel('Years'),ylabel('Haircut, \tau_t'), 
% title('(c)')
% legend('median','Location','SouthEast')
% box on

% subplot(2,2,4)
% plot(ss,pip)
% xlabel('State (s)'),ylabel('Probability') , title('(d)')

xout=[xout x];
% ssout=[ssout ss];
% pipout=[pipout pip];
spathout=[spathout spath(:,end)];
xpathout=[xpathout xpath(:,end)];
resout=[resout resid];
vout=[vout v];
tauanaout=[tauanaout tauana'];
end
end

% show analytical solution against optimal
figure
subplot(1,2,1), hold all
plot(s,xout(:,1),'b','Linewidth',2.5)
plot(s,tauanaout(:,1),':r','Linewidth',2.5)
title('Without liquidation')
legend('Numerical','Analytical')
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12)
ylabel('Optimal haircut, $\tau_t (k_t)$','Interpreter','latex','Fontsize',12)
box on
subplot(1,2,2), hold all
plot(s,xout(:,3),'b','Linewidth',2.5)
plot(s,tauanaout(:,3),':r','Linewidth',2.5)
legend('Numerical','Analytical')
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12)
ylabel('Optimal haircut, $\tau_t (k_t)$','Interpreter','latex','Fontsize',12)
box on
title('With liquidation')

% show optimal policy as a fn of capital, w and wo liquidation
figure, hold all,
plot(s,xout(:,1),'b','LineWidth',2.5)
plot(s,xout(:,3),':b','LineWidth',2.5)
legend('Without liquidation','With liquidation','Location','NorthEast')
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12)
ylabel('Optimal haircut, $\tau_t (k_t)$','Interpreter','latex','Fontsize',12)
box on

% show optimal policy as a fn of capital, w and wo trap, with liquidation
figure, hold all, 
plot(s,xout(:,3),'b','LineWidth',2.5)
plot(s,xout(:,4),':r','LineWidth',2.5)
legend('Without credit trap','With credit trap','Location','NorthEast')
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12)
ylabel('Optimal haircut, $\tau_t (k_t)$','Interpreter','latex','Fontsize',12)
box on

% plot details of the numerics
wid=2.5;
for j=1:4
figure,
subplot(2,2,1)
plot(s,vout(:,j),'-b','LineWidth',wid), axis tight, box on
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12),
ylabel('$V(k_t , \tau_t (k_t))$','Interpreter','latex','Fontsize',12), 
title('(a) Value function')

subplot(2,2,2), hold all
plot(s,xout(:,j),'-b','LineWidth',wid), 
plot(s,zeros(1,length(s)),'-k'), 
plot(s,ones(1,length(s)),'-k'), 
axis tight, box on
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12),
ylabel('$\tau_t (k_t)$','Interpreter','latex','Fontsize',12),
title('(b) Optimal haircut')

subplot(2,2,3)
plot(s,resout(:,j),'-b','LineWidth',wid), axis tight, box on
xlabel('Capital, $k_t$','Interpreter','latex','Fontsize',12),
title('(c) Residuals')
end

% plot densities
fxi=[];
xi=[];
fsi=[];
si=[];
for i=1:4
   [fsi(:,i),si(:,i)]= ksdensity(spathout(:,i));
   [fxi(:,i),xi(:,i)]= ksdensity(xpathout(:,i));
end

df=2.5;

figure,
hold all
plot(si(:,1),fsi(:,1),'b','LineWidth',df)
plot(si(:,3),fsi(:,3),':r','LineWidth',df)
plot(si(:,4),fsi(:,4),'--g','LineWidth',df)
ylabel('Density','Interpreter','latex','FontSize',12)
xlabel('Capital, $k_t$','Interpreter','latex','FontSize',12)
box on
legend('w/o liquidation','w/ liquidation','w/ liquidation and trap',...
    'Location','NorthOutside','Orientation','horizontal')

figure,
hold all
plot(xi(:,1),fxi(:,1),'b','LineWidth',df)
plot(xi(:,3),fxi(:,3),':r','LineWidth',df)
plot(xi(:,4),fxi(:,4),'--g','LineWidth',df)
ylabel('Density','Interpreter','latex','FontSize',12)
xlabel('Haircut, $\tau_t (k_t)$','Interpreter','latex','FontSize',12)
box on
legend('w/o liquidation','w/ liquidation','w/ liquidation and trap',...
    'Location','NorthOutside','Orientation','horizontal')


kskews=skewness(spathout)
tauskews=skewness(xpathout)

kkurt=kurtosis(spathout)
taukurt=kurtosis(xpathout)