function [out1,out2,out3] = traps9(flag,k,tau,e,alpha,beta,pi,lambda0,lambda1,lambda2,Exalpha,lambdab,xb,liq,trap)
% adds liquidation to the constrained policy case in traps5

switch flag
    
    case 'b';
        
        if trap==1
        
            tmin=[];
            options=optimset('Display','off');
                for j=1:length(k)
                    ll=lambda0+lambda1.*(k(j)./(lambda2+k(j)));    
                    zz = fsolve(@(t) ((pi+beta.*(1-t).*ll).^alpha).*((1+beta.*(1-t).*ll).^(1-alpha)).*Exalpha ...
                           -((pi+beta.*(1-t).*lambdab).^alpha).*((1+beta.*(1-t).*lambdab).^(1-alpha)).*(xb^alpha),0.2,options) ;
                    tmin=[tmin zz];
                end
            out1 = max( 0 , tmin' )  ;
        
        else
            
            out1 = 0.0*ones(size(k))   ; % lower bound on the instrument    
        
        end
        
            out2 = 1.0*ones(size(k))       ; % upper bound on the instrument
        
    case 'f'; % reward function and its first and second derives wrt tau
        
        ll=lambda0+lambda1.*(k./(lambda2+k));
        
        % payoff
        out1 = alpha*(1+alpha*beta)*log(k) - (1+alpha*beta).*log(1 + beta.*(1-tau).*ll) + alpha*beta*log(pi + beta.*(1-tau).*ll);
        
        % 1st deriv
        d1 = (alpha.*beta)./(pi + beta.*(1-tau).*ll) ;
        d2 = (1+alpha.*beta)./(1+ beta.*(1-tau).*ll) ;
        out2 = -beta.*ll.*( d1 - d2 ) ;
        
        % 2nd deriv
        dd1 = (1+alpha.*beta)./((1 + beta.*(1-tau).*ll ).^2) ;
        dd2 = alpha.*beta ./ ((pi + beta.*(1-tau).*ll).^2) ;
        out3 = ((beta.*ll).^2).*(dd1 - dd2); 
        
    case 'g'; % law of motion and derivatives wrt action
        
        % CASE OF LIQUIDATION
        if liq==1
            
        ll=lambda0+lambda1.*(k./(lambda2+k));
        
            % good shock realisation:
            if e>Exalpha
        
            shk=e;
            
            % law of motion
            out1 = shk.*((pi+beta.*(1-tau).*ll)./(1+beta.*(1-tau).*ll)).*(1-alpha).*(k.^alpha) ;
        
            % 1st deriv
            f1 = - beta.*ll.*(1-pi) ./ (( 1 + beta.*(1-tau).*ll ).^2) ;
            out2 = shk.*f1.*(1-alpha).*(k.^alpha);
        
            % 2nd deriv
            ff1 = ((beta*ll).^2) ./ ((1 + beta.*(1-tau).*ll).^3) ;
            out3 = -2.*shk.*ff1.*(1-pi).*(1-alpha).*(k.^alpha);
        
            % bad shock realisation:
            else

            shk=( ( e.^alpha - (1-tau).*ll.*Exalpha)./(1-(1-tau).*ll) ).^(1/alpha);

            % law of motion
            out1 = shk.*((pi+beta.*(1-tau).*ll)./(1+beta.*(1-tau).*ll)).*(1-alpha).*(k.^alpha) ;

            % 1st deriv
            f1 = - beta.*ll.*(1-pi) ./ (( 1 + beta.*(1-tau).*ll ).^2) ;

            g1 = ll.*( Exalpha - e.^alpha )./alpha ;
            g2 = ( e.^alpha - (1-tau).*ll.*Exalpha ).^((1-alpha)./alpha);
            g3 = (1 - (1-tau).*ll).^((1+alpha)/alpha);
            dg = g1.*g2./g3;

            out2 = shk.*f1.*(1-alpha).*(k.^alpha) + dg.*((pi+beta.*(1-tau).*ll)./(1+beta.*(1-tau).*ll)).*(1-alpha).*(k.^alpha);

            % 2nd deriv
            ff1 = ((beta*ll).^2) ./ ((1 + beta.*(1-tau).*ll).^3) ;

            dg2 = dg.*(ll./alpha).*((1-alpha).*Exalpha./(g2.^(alpha/(1+alpha))) - (1+alpha).*(g3.^(alpha/(1+alpha))));

            out3 = -2.*shk.*ff1.*(1-pi).*(1-alpha).*(k.^alpha) ...
                  + 2.*dg.*f1.*(1-alpha).*(k.^alpha) ...
                  + dg2.*((pi+beta.*(1-tau).*ll)./(1+beta.*(1-tau).*ll)).*(1-alpha).*(k.^alpha);        
        
            end
        
        % CASE OF NO LIQUIDATION:
        else
            
            ll=lambda0+lambda1.*(k./(lambda2+k));

            shk=e;

            % law of motion
            out1 = shk.*((pi+beta.*(1-tau).*ll)./(1+beta.*(1-tau).*ll)).*(1-alpha).*(k.^alpha) ;

            % 1st deriv
            f1 = - beta.*ll.*(1-pi) ./ (( 1 + beta.*(1-tau).*ll ).^2) ;
            out2 = shk.*f1.*(1-alpha).*(k.^alpha);

            % 2nd deriv
            ff1 = ((beta*ll).^2) ./ ((1 + beta.*(1-tau).*ll).^3) ;
            out3 = -2.*shk.*ff1.*(1-pi).*(1-alpha).*(k.^alpha);
        
        end


end
